<?php namespace EscapeWork\Manager\News\Models;

use EscapeWork\LaravelHelpers\BaseModel;
use EscapeWork\LaravelHelpers\SluggableTrait;
use EscapeWork\Manager\Medias\Helpers\Mediable;
use Carbon\Carbon;
use InvalidArgumentException;

class News extends BaseModel implements Mediable
{

    /**
     * Tabela
     */
    protected $table = 'news';

    /**
     * Sluggable
     */
    use SluggableTrait;

    /**
     * Dates array
     */
    protected $dates = ['date'];

    /**
     * Regras de validação
     */
    public $validationRules = array(
        'title'       => array('required'),
        'description' => array('required'),
        'date'        => array('required'),
    );

    /**
     * Mensagens de validação
     */
    public $validationMessages = array(
        'slug.required'        => 'O slug é obrigatório',
        'description.required' => 'A descrição é obrigatória',
        'date.required'        => 'A data é obrigatória',
    );

    /**
     * Fillable fields
     */
    protected $fillable = array(
        'news_category_id',
        'title',
        'subtitle',
        'description',
        'featured',
        'active',
        'date',
        'source',
    );

    public function setDateAttribute($value)
    {
        $this->_setDateAttribute('date', $value);
    }

    public function setFeaturedAttribute($value)
    {
        $this->attributes['featured'] = $value == 1 ? 1 : 0;

        if (! $this->attributes['featured']) {
            $this->attributes['order'] = null;
        } else {
            $this->attributes['order'] = $this->getLastOrder();
        }
    }

    public function medias()
    {
        return $this->morphMany('EscapeWork\Manager\Medias\Models\Media', 'medias', 'media_model', 'model_id')
                    ->orderBy('order');
    }

    public function scopeActive($query)
    {
        return $query->where('active', '=', 1);
    }

    public function scopeFeatured($query)
    {
        return $query->where('featured', '=', 1);
    }

    public function scopeJoinCategories($query)
    {
        return $query->join('news_categories', 'news_categories.id', '=', 'news.news_category_id');
    }

    public function scopeLeftJoinCategories($query)
    {
        return $query->leftJoin('news_categories', 'news_categories.id', '=', 'news.news_category_id');
    }

    public function delete()
    {
        $this->deleteMedias();

        return parent::delete();
    }

    protected function deleteMedias()
    {
        foreach ($this->medias as $media) {
            $media->delete();
        }
    }

    public function getLastOrder()
    {
        $news = $this->orderBy('order', 'desc')->take(1)->first();

        return $news ? $news->order + 1 : 1;
    }

    public function getMediaModel()
    {
        return 'EscapeWork\Manager\News\Models\News';
    }

    public function getExcerptAttribute()
    {
        return trim(strip_tags($this->attributes['description']));
    }
}
